<?php

namespace SGalinski\ContentReplacer\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Stefan Galinski <stefan.galinski@gmail.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;

/**
 * Repository for fetching terms
 *
 * @author Stefan Galinski <stefan.galinski@gmail.com>
 * @package TYPO3
 * @subpackage content_replacer
 */
class TermRepository {
	/**
	 * Returns the given terms with their related information's.
	 *
	 * @param array $filterTerms
	 * @param string $category
	 * @return array
	 */
	public function fetchTerms(array $filterTerms, $category) {
		/** @var DatabaseConnection $database */
		$database = $GLOBALS['TYPO3_DB'];
		/** @var TypoScriptFrontendController $typoScriptFrontendController */
		$typoScriptFrontendController = $GLOBALS['TSFE'];

		$termsWhereClause = [];
		$category = $database->fullQuoteStr($category, 'tx_content_replacer_category');
		foreach ($filterTerms as $term) {
			$termsWhereClause[] = $database->fullQuoteStr(trim($term), 'tx_content_replacer_term');
		}

		$queryResource = $database->exec_SELECTquery(
			'tx_content_replacer_term.uid, tx_content_replacer_term.pid, ' .
			'term, replacement, stdWrap, category_uid, sys_language_uid',
			'tx_content_replacer_term, tx_content_replacer_category',
			'term IN (' . implode(', ', $termsWhereClause) . ') AND ' .
			'sys_language_uid IN (-1, 0) AND category = ' . $category . ' AND ' .
			'tx_content_replacer_category.uid = category_uid ' .
			$typoScriptFrontendController->cObj->enableFields('tx_content_replacer_term') . ' ' .
			$typoScriptFrontendController->cObj->enableFields('tx_content_replacer_category')
		);

		$terms = [];
		$languageMode = $typoScriptFrontendController->sys_language_uid;
		$overlayMode = $typoScriptFrontendController->sys_language_contentOL;
		while ($term = $database->sql_fetch_assoc($queryResource)) {
			if ($languageMode) {
				$term = $typoScriptFrontendController->sys_page->getRecordOverlay(
					'tx_content_replacer_term', $term, $languageMode, $overlayMode
				);
			}

			$terms[$term['term']] = $term;
		}
		$database->sql_free_result($queryResource);

		return $terms;
	}
}